#include "EdgeDetectors.h"

Comrade::IrisXT::EdgeDetectorManager::EdgeDetectorManager()
{
    cout<<"EdgeDetectorManager object created...\n";
}

double Comrade::IrisXT::EdgeDetectorManager::greyscale_correlate(Buffer<RGB>* buffer,
                                                int x,int y,
                                                KernelMatrix* ker)
{
    double sum=0;
    int yhalf=(ker->ysize-1)/2;
    int xhalf=(ker->xsize-1)/2;

    for (int yy=-yhalf; yy<=yhalf; yy++)
    {
        for (int xx=-xhalf; xx<=xhalf; xx++)
        {
            if (xx+x>=0 && xx+x<=buffer->maxx-1 &&
                yy+y>=0 && yy+y<=buffer->maxy-1)
            {
                sum+=buffer->at(xx+x,yy+y).green*
                    (ker->matrix[yy+yhalf][xx+xhalf]);
            }
        }
    }

    return sum;
}

void Comrade::IrisXT::EdgeDetectorManager::call_operator(KernelMatrix* H1,
                                        KernelMatrix* H2,
                                        int threshold)
{
    for (int y=0; y<=ip->maxy-1; y++)
    {
        for (int x=0; x<=ip->maxx-1; x++)
        {
            double value1=greyscale_correlate(ip,x,y,H1);
            double value2=greyscale_correlate(ip,x,y,H2);

            double final=fabs(value1)+fabs(value2);
            final/=2;

            final=(final>255)?255:final;
            final=(final<0)?0:final;

            //op->at(x,y).red=static_cast<unsigned char>(final);
            //op->at(x,y).green=static_cast<unsigned char>(final);
            //op->at(x,y).blue=static_cast<unsigned char>(final);

            if (final>=threshold)
            {
                op->at(x,y).red=255;
                op->at(x,y).green=255;
                op->at(x,y).blue=255;
            }

            else
            {
                op->at(x,y).red=0;
                op->at(x,y).green=0;
                op->at(x,y).blue=0;
            }
        }
    }
}

void Comrade::IrisXT::EdgeDetectorManager::RobertsCross(Buffer<RGB>* ip_buffer,
                                       Buffer<RGB>* op_buffer,
                                       int threshold)
{
    ip=ip_buffer;
    op=op_buffer;

    KernelMatrix H1(3,3);
    KernelMatrix H2(3,3);

    KernelGenerator::generate_RobertsCross(&H1,&H2);

    /*double arr1[]={ 0, 0, 0,
                    0, 0, 1,
                    0,-1, 0};

    double arr2[]={ 0, 0, 0,
                    0, 1, 0,
                    0, 0,-1};*/

    call_operator(&H1,&H2,threshold);
}

void Comrade::IrisXT::EdgeDetectorManager::Prewitt(Buffer<RGB>* ip_buffer,
                                  Buffer<RGB>* op_buffer,
                                  int threshold)
{
    ip=ip_buffer;
    op=op_buffer;

    KernelMatrix H1(3,3);
    KernelMatrix H2(3,3);

    KernelGenerator::generate_Prewitt(&H1,&H2);

    /*double arr1[]={-1, 0, 1,
                   -1, 0, 1,
                   -1, 0, 1};

    double arr2[]={-1,-1,-1,
                    0, 0, 0,
                    1, 1, 1};*/

    call_operator(&H1,&H2,threshold);
}

void Comrade::IrisXT::EdgeDetectorManager::Sobel(Buffer<RGB>* ip_buffer,
                                Buffer<RGB>* op_buffer,
                                int threshold)
{
    ip=ip_buffer;
    op=op_buffer;

    KernelMatrix H1(3,3);
    KernelMatrix H2(3,3);

    KernelGenerator::generate_Sobel(&H1,&H2);

    /*double arr1[]={-1, 0, 1,
                   -2, 0, 2,
                   -1, 0, 1};

    double arr2[]={-1,-2,-1,
                    0, 0, 0,
                    1, 2, 1};*/

    call_operator(&H1,&H2,threshold);
}

void Comrade::IrisXT::EdgeDetectorManager::isotropic(Buffer<RGB>* ip_buffer,
                                    Buffer<RGB>* op_buffer,
                                    int threshold)
{
    ip=ip_buffer;
    op=op_buffer;

    KernelMatrix H1(3,3);
    KernelMatrix H2(3,3);

    KernelGenerator::generate_isotropic(&H1,&H2);

    /*double arr1[]={-1, 0, 1,
                   -1.414, 0, 1.414,
                   -1, 0, 1};

    double arr2[]={-1,-1.414,-1,
                    0, 0, 0,
                    1, 1.414, 1};*/

    call_operator(&H1,&H2,threshold);
}

void Comrade::IrisXT::EdgeDetectorManager::Laplacian(Buffer<RGB>* ip_buffer,
                                    Buffer<RGB>* op_buffer,
                                    int threshold)
{
    ip=ip_buffer;
    op=op_buffer;

    KernelMatrix H1(3,3);
    KernelMatrix H2(3,3);

    KernelGenerator::generate_Laplacian(&H1,&H2);

    /*double arr1[]={ 1,-2, 1,
                   -2, 4,-2,
                   -1,-2, 1};

    double arr2[]={ 0, 0, 0,
                    0, 0, 0,
                    0, 0, 0};*/

    call_operator(&H1,&H2,threshold);
}

Comrade::IrisXT::EdgeDetectorManager::~EdgeDetectorManager()
{}


